from .api_base_model import ApiBaseModel
from .code import Code

class Address(ApiBaseModel):
  addressId: str
  text: str
  line: str
  city: str
  district: str
  state: str
  postalCode: str
  country: Code

  @classmethod
  def add_address(cls, id, line, city, district, state, postal_code, country):
    text = "%s, %s, %s, %s, %s, %s" % (line, city, district, state, postal_code, country.decode)
    text = text.replace(' ,', '')
    return Address(addressId=id, text=text, line=line, city=city, district=district, state=state, postalCode=postal_code, country=country)


import pytest

from src.usdm_excel.errors.errors import Errors

def test_create():
  errors = Errors()
  assert errors.items == []

def test_add():
  errors = Errors()
  assert len(errors.items) == 0
  errors.add(sheet="My Sheet", row=1, column=99, message="XXXXX")
  assert len(errors.items) == 1
  assert errors.items[0].message == "XXXXX"
    
def test_count():
  errors = Errors()
  errors.add(sheet="My Sheet", row=1, column=99, message="XXXXX")
  errors.add(sheet="My Sheet", row=1, column=99, message="XXXXX")
  assert errors.count() == 2



from usdm_excel.base_sheet import BaseSheet
from usdm_excel.cross_ref import cross_references
from usdm_excel.id_manager import id_manager
import traceback
import pandas as pd
from usdm_model.objective import Objective
from usdm_model.endpoint import Endpoint

class StudyDesignObjectiveEndpointSheet(BaseSheet):

  def __init__(self, file_path):
    try:
      #super().__init__(pd.read_excel(open(file_path, 'rb'), sheet_name='studyDesignOE'))
      super().__init__(file_path=file_path, sheet_name='studyDesignOE')
      self.objectives = []
      current = None
      for index, row in self.sheet.iterrows():
        o_xref = self.read_cell_by_name(index, "objectiveXref")
        o_description = self.read_cell_by_name(index, "objectiveDescription")
        o_level = self.read_cdisc_klass_attribute_cell_by_name('Objective', 'objectiveLevel', index, "objectiveLevel")
        e_xref = self.read_cell_by_name(index, "endpointXref")
        e_description = self.read_cell_by_name(index, "endpointDescription")
        e_p_description = self.read_cell_by_name(index, "endpointPurposeDescription")
        e_level = self.read_cdisc_klass_attribute_cell_by_name('Endpoint', 'endpointLevel', index, "endpointLevel")
        if not o_description == "":
          current = Objective(objectiveId=id_manager.build_id(Objective),
            objectiveDescription=o_description, 
            objectiveLevel=o_level,
            objectiveEndpoints=[]
          )
          self.objectives.append(current)
          cross_references.add(o_xref, current.objectiveId)
        ep = Endpoint(endpointId=id_manager.build_id(Endpoint), 
          endpointDescription=e_description, 
          endpointPurposeDescription=e_p_description, 
          endpointLevel=e_level
        )  
        current.objectiveEndpoints.append(ep)
        cross_references.add(e_xref, ep.endpointId)
        
    except Exception as e:
      print("Oops!", e, "occurred.")
      traceback.print_exc()



from typing import Union
from .api_base_model import ApiBaseModel
from .code import Code
from uuid import UUID

# class PreviousProcedure(ApiBaseModel):
#   procedureName: str
#   procedureCode: Union[Code, UUID]

class Procedure(ApiBaseModel):
  procedureId: str
  procedureType: str
  procedureCode: Code
  procedureIsConditional: bool
  procedureIsConditionalReason: str



import pytest

from src.usdm_excel.errors.error import Error

def test_create():
    error = Error(sheet="My Sheet", row=1, column=99, message="XXXXX")
    assert error.sheet == "My Sheet"
    assert error.row == 1
    assert error.column == 99
    assert error.message == "XXXXX"
